/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is opencrx.
 *
 * The Initial Developer of the Original Code is
 * Josh Geenen <opencrx@pirules.net>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
/**
 * oOverlay.js
 * Contains the oOverlay class and a load listener for the Address Book.
 * When the Address Book loads, initializes the string bundle(s), gets the
 * preferences, initializes the oFileIO class and member files, and
 * checks for an authentication token.  If there is no auth token it prompts the
 * user to login.
 */
// initialize everything after the Address Book window loads
window.addEventListener("load", function eventListener(e) { oOverlay.initialize(); }, false);
var testUrl = "http://guest:guest@demo.opencrx.org/opencrx-vcard-CRX/" +
              "vcard?id=CRX/Standard/filter/josh&resource=accounts.vcf";
/**
 * oOverlay
 * Checks if the authentication token is present and valid.  If so, it starts
 * everything up and synchronizes the contacts.  Otherwise it shows the
 * login window.
 * @class
 */
var oOverlay = {
  /**
   * Called when the overlay is loaded and initializes everything and begins
   * the authentication check and sync or login prompt.
   */
  initialize: function oOverlay_initialize() {
    // determine if this is before or after Bug 413260 landed
    var card = Cc["@mozilla.org/addressbook/cardproperty;1"]
              .createInstance(nsIAbCard);
    this.mBug413260 = card.getProperty ? true : false;
    oFileIO.init(); // initialize the oFileIO class
    if (oFileIO.mLogFile && oFileIO.mLogFile.exists())
      oFileIO.mLogFile.remove(false); // delete the old log file
    // log some basic system and application info
    oLOGGER.LOG("Loading openCRX extension at " + new Date());
    oLOGGER.LOG(" * Version is: " + oVersion);
    oLOGGER.LOG(" * User Agent: " + navigator.userAgent + "\n");
    oPreferences.getSyncPrefs(); // get the preferences
    if (oPreferences.mSyncPrefs.enableBtns.value)
      this.setupButton(); // add the import and export buttons
    if (oPreferences.mSyncPrefs.enableMenu.value)
      this.setupMenu();

    /**/
    //oExport.begin(testUrl, "test");
    /*
    oAbListener.add(); // add the address book listener
    // call the unload function when the address book window is shut
    window.addEventListener("unload", function unloadListener(e) { oOverlay.unload(); }, false);
   */
  },
  startImport: function oOverlay_startImport() {
    var dir = GetSelectedDirectory();
    var ab;
    if (dir) {
      try {
      ab = new oAddressBook(oAbManager.getAbByURI(dir));
      } catch (e) { return; }
    }
    if (!ab) return;
    
    if (!ab.getUrl()) {
      var prompt = Cc["@mozilla.org/embedcomp/prompt-service;1"]
                .getService(Ci.nsIPromptService)
                .prompt;
      var url = {};

      var ok = prompt(window, oStringBundle.getStr("addAbUrlTitle")
                      + ab.getName(),
                      oStringBundle.getStr("addAbUrl"), url, null,
                      {value: false});
      if (!ok || !url || url.value.length < 10)
        return;
      ab.setUrl(url.value);
    }

    var req = new oHttpRequest();
    req.mUrl = ab.getUrl();
    req.mType = "GET";
    req.mOnSuccess = ["oImport.begin(httpReq.responseText, '" + ab.getName() + "');"];
    req.mOnError = ["alert(httpReq.responseText);"];
    oOverlay.setStatusBarText("Sending request for vcards");
    req.send();

  },
  startExport: function oOverlay_startExport() {
    //alert("sorry, this feature is not complete yet.\nJosh");
    var dir = GetSelectedDirectory();
    var ab;
    if (dir) {
      try {
        ab = new oAddressBook(oAbManager.getAbByURI(dir));
      } catch (e) { return; } // could get the AB
    }
    if (!ab) return;
    
    if (!ab.getUrl()) {
      var prompt = Cc["@mozilla.org/embedcomp/prompt-service;1"]
                .getService(Ci.nsIPromptService)
                .prompt;
      var url = {};

      // opens a username/password prompt
      var ok = prompt(window, oStringBundle.getStr("addAbUrlTitle")
                      + ab.getName(),
                      oStringBundle.getStr("addAbUrl"), url, null,
                      {value: false});
      if (!ok || !url || url.value.length < 10)
        return;
      ab.setUrl(url.value);
    }

    oExport.begin(ab);
  },
  /**
   * oOverlay.setupButton
   * Sets up the Sync button to go between the Write and Delete buttons and adds
   * a separator between Sync and Delete.
   */
  setupButton: function oOverlay_setupButton() {
    // get the toolbar with the buttons
    try {
      oLOGGER.VERBOSE_LOG("Trying to add button");
      // get the toolbar with the buttons
      var toolbar     = document.getElementById("ab-bar2");   // thunderbird
      var isSeamonkey = toolbar ? true : false;
      if (!toolbar) {
        oLOGGER.VERBOSE_LOG("Didn't find ab-bar2...looking for abToolbar")
        toolbar       = document.getElementById("abToolbar"); // seamonkey
        if (!toolbar) {
          oLOGGER.LOG_ERROR("Could not find the toolbar");
          return false;
        }
      }
      var deleteButton = document.getElementById(isSeamonkey ? "button-abdelete" : "button-delete");
      // setup the separator
      var separator = document.createElement("toolbarseparator");
      separator.setAttribute("id", "new-separator1");
      // setup the button
      var button = document.createElement("toolbarbutton");
      button.setAttribute("class", "openCRX-Button toolbarbutton-1" + 
                          " chromeclass-toolbar-additional");
      button.setAttribute("id", "oImportButton");
      button.setAttribute("label", oStringBundle.getStr("importButton"));
      button.setAttribute("oncommand", "oOverlay.startImport();");
      button.setAttribute("tooltiptext", oStringBundle.getStr("importTooltip"));
      button.setAttribute("insertbefore", "new-separator1");

      button.style.MozImageRegion = deleteButton.MozImageRegion;
      // insert the separator before the Delete button
      toolbar.insertBefore(separator, deleteButton);
      // insert the button before the separator
      toolbar.insertBefore(button, separator);
      if (oPreferences.mSyncPrefs.forceBtnImage.value) {
        oLOGGER.VERBOSE_LOG("Forcing the listStyleImage for the import button");
        document.getElementById("oImportButton").style.listStyleImage =
          "url('chrome://opencrx/skin/import.png')";
      }
    }
    catch(e) { oLOGGER.LOG_WARNING("Couldn't setup the import button", e); }
    // Export doesn't work before TB 3
    if (oAbManager.mVersion < 3)
      return true;
    try {
      // setup the separator
      var separator = document.createElement("toolbarseparator");
      separator.setAttribute("id", "new-separator2");
      // setup the button
      var button = document.createElement("toolbarbutton");
      button.setAttribute("class", "openCRX-Button toolbarbutton-1" + 
                          " chromeclass-toolbar-additional");
      button.setAttribute("id", "oExportButton");
      button.setAttribute("label", oStringBundle.getStr("exportButton"));
      button.setAttribute("oncommand", "oOverlay.startExport();");
      button.setAttribute("tooltiptext", oStringBundle.getStr("exportTooltip"));
      button.setAttribute("insertbefore", "new-separator2");

      button.style.MozImageRegion = deleteButton.MozImageRegion;
      // insert the separator before the Delete button
      toolbar.insertBefore(separator, deleteButton);
      // insert the button before the separator
      toolbar.insertBefore(button, separator);
      if (oPreferences.mSyncPrefs.forceBtnImage.value) {
        oLOGGER.VERBOSE_LOG("Forcing the listStyleImage for the export button");
        document.getElementById("oExportButton").style.listStyleImage =
          "url('chrome://opencrx/skin/export.png')";
      }
    }
    catch(e) { oLOGGER.LOG_WARNING("Couldn't setup the export button", e); }
    return true;
  },
  /**
   * oOverlay.setupMenu
   * Sets up the openCRX menu in the address book menubar
   */
  setupMenu: function oOverlay_setupMenu() {
    try {
      var menubar      = document.getElementById("mail-menubar");
      var isSeamonkey  = menubar ? true : false;
      if (!menubar) // seamonkey
          menubar      = document.getElementById("ab-menubar");

      var toolsMenu    = document.getElementById("tasksMenu");
      var menu         = document.createElement("menu");
      menu.setAttribute("id", "openCRXMenu");
      menu.setAttribute("label", "openCRX");
      menu.setAttribute("accesskey", "o");
      var menupopup    = document.createElement("menupopup");
      menupopup.setAttribute("id", "openCRXMenuPopup");

      var importMenuItem = document.createElement("menuitem");
      importMenuItem.setAttribute("id", "importMenuItem");
      importMenuItem.setAttribute("label", oStringBundle.getStr("importMenu"));
      importMenuItem.setAttribute("accesskey", oStringBundle.getStr("importMenuKey"));
      importMenuItem.setAttribute("oncommand", "oOverlay.startImport();");
      importMenuItem.setAttribute("class", "menuitem-iconic icon-mail16 menu-iconic");

      if (oAbManager.mVersion >= 3) {
        var exportMenuItem = document.createElement("menuitem");
        exportMenuItem.setAttribute("id", "importMenuItem");
        exportMenuItem.setAttribute("label", oStringBundle.getStr("exportMenu"));
        exportMenuItem.setAttribute("accesskey", oStringBundle.getStr("exportMenuKey"));
        exportMenuItem.setAttribute("oncommand", "oOverlay.startExport();");
        exportMenuItem.setAttribute("class", "menuitem-iconic icon-mail16 menu-iconic");
      }

      var prefMenuItem = document.createElement("menuitem");
      prefMenuItem.setAttribute("id", "prefMenuItem");
      prefMenuItem.setAttribute("label", oStringBundle.getStr("prefMenu"));
      prefMenuItem.setAttribute("accesskey", oStringBundle.getStr("prefMenuKey"));
      prefMenuItem.setAttribute("oncommand", "oOverlay.openPreferences();");
      prefMenuItem.setAttribute("class", "menuitem-iconic icon-mail16 menu-iconic");

      var forumMenuItem = document.createElement("menuitem");
      forumMenuItem.setAttribute("id", "forumMenuItem");
      forumMenuItem.setAttribute("label", oStringBundle.getStr("forumMenu"));
      forumMenuItem.setAttribute("accesskey", oStringBundle.getStr("forumMenuKey"));
      forumMenuItem.setAttribute("oncommand", "oOverlay.openURL('extensions.opencrx.forumURL');");
      forumMenuItem.setAttribute("class", "menuitem-iconic icon-mail16 menu-iconic");

      var wikiMenuItem = document.createElement("menuitem");
      wikiMenuItem.setAttribute("id", "wikiMenuItem");
      wikiMenuItem.setAttribute("label", oStringBundle.getStr("wikiMenu"));
      wikiMenuItem.setAttribute("accesskey", oStringBundle.getStr("wikiMenuKey"));
      wikiMenuItem.setAttribute("oncommand", "oOverlay.openURL('extensions.opencrx.wikiURL');");
      wikiMenuItem.setAttribute("class", "menuitem-iconic icon-mail16 menu-iconic");

      var errorMenuItem = document.createElement("menuitem");
      errorMenuItem.setAttribute("id", "errorMenuItem");
      errorMenuItem.setAttribute("label", oStringBundle.getStr("errorMenu"));
      errorMenuItem.setAttribute("accesskey", oStringBundle.getStr("errorMenuKey"));
      errorMenuItem.setAttribute("oncommand", "oOverlay.openURL('extensions.opencrx.errorURL');");
      errorMenuItem.setAttribute("class", "menuitem-iconic icon-mail16 menu-iconic");

      menupopup.appendChild(importMenuItem);
      if (oAbManager.mVersion >= 3)
        menupopup.appendChild(exportMenuItem);
      menupopup.appendChild(prefMenuItem);
      menupopup.appendChild(forumMenuItem);
      menupopup.appendChild(errorMenuItem);
      menupopup.appendChild(wikiMenuItem);
      menu.appendChild(menupopup);
      menubar.insertBefore(menu, toolsMenu);
    }
    catch(e) {
      oLOGGER.LOG_WARNING("Unable to setup the menu", e);
    }
  },  /**
   * Called when the overlay is unloaded and removes the address book listener.
   */
  unload: function oOverlay_unload() {
    oAbListener.remove();
  },
  /**
   * oOverlay.setStatusBarText
   * Sets the text of the status bar to the given value.
   * @param aText  The text to put on the status bar.
   */
  setStatusBarText: function oOverlay_setStatusBarText(aText) {
    document.getElementById("oStatusText2").label = aText;
  },
  /**
   * oOverlay.getStatusBarText
   * Gets the text of the status bar.
   * @return The text of the status bar
   */
  getStatusBarText: function oOverlay_getStatusBarText(aText) {
    return document.getElementById("oStatusText2").label;
  },
  /**
   * oOverlay.writeTimeToStatusBar
   * Writes the current time to the status bar along with the sync finished
   * string.
   * When the status text is clicked the log file is opened.
   */
  writeTimeToStatusBar: function oOverlay_writeTimeToStatusBar() {
    var hours = new String(new Date().getHours());
    hours = hours.length == 0 ? "00" + hours : hours;
    hours = hours.length == 1 ? "0" + hours : hours;
    var minutes = new String(new Date().getMinutes());
    minutes = minutes.length == 1 ? "0" + minutes : minutes;
    var seconds = new String(new Date().getSeconds());
    seconds = seconds.length == 1 ? "0" + seconds : seconds;
    var text = oStringBundle.getStr("syncFinishedString");
    this.setStatusBarText(text + " " + hours + ":" + minutes + ":" + seconds);
    document.getElementById("oStatusText2").tooltip = "test";
  },
  /**
   * oOverlay.showLog
   * Opens the "view source" window with the log file.
   */
  showLog: function oOverlay_showLog() {
    try {
      var windowFeatures = "chrome=yes,resizable=yes,height=400,width=600";
      window.open("view-source:file://" + oFileIO.mLogFile.path,
                  "Log", windowFeatures);
    }
    catch(e) {
      oLOGGER.LOG_WARNING("Unable to open the log", e);
    }
  },
  /**
   * oOverlay.openPreferences
   * Opens the oPreferences for opencrx
   */
  openPreferences: function oOverlay_openPreferences() {
    var win = window.open("chrome://opencrx/content/options.xul", "prefs",
                          "chrome=yes,resizable=yes,toolbar=yes,centerscreen=yes");
    // when the pref window loads, set its onunload property to get the prefs again
    win.onload = function onloadListener() {
      win.onunload = function onunloadListener() {
        try { oPreferences.getSyncPrefs(); } catch (e) {}
      };
    };    
  },
  /**
   * openURL
   * Opens the given URL using the openFormattedURL and
   * openFormattedRegionURL functions.
   *
   * @param aURL {string} THe URL to open.
   */
  openURL: function oOverlay_openURL(aURL) {
    oLOGGER.VERBOSE_LOG("Opening the following URL: " + aURL);
    if (!aURL) {
      oLOGGER.LOG_WARNING("Caught an attempt to load a blank URL");
      return;
    }
    try {
      if (openFormattedURL) {
        openFormattedURL(aURL);
        return;
      }
    }
    catch (e) {}
    try {
      if (openFormattedRegionURL) {
        openFormattedRegionURL(aURL);
        return;
      }
    }
    catch (e) {}
    oLOGGER.LOG_WARNING("Could not open the URL: " + aURL);
    return;
  }
};
